/*
 * Copyright (C) 2008 Felipe Contreras <felipe.contreras@gmail.com>
 * Copyright (C) 2010 Nicola Murino <n.murino@prassel.it>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02111-1301  USA
 */

#ifndef GST_OMAPFB_H
#define GST_OMAPFB_H

#include <gst/gst.h>
#include <gst/video/gstvideosink.h>
#include <gst/video/video.h>

#include <X11/Xlib.h>
#include <X11/Xutil.h>

#include <linux/fb.h>
#include <linux/omapfb.h>

G_BEGIN_DECLS

#define GST_OMAPFB_SINK_TYPE (gst_omapfbsink_get_type ())
#define GST_OMAPFB_SINK(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GST_OMAPFB_SINK_TYPE, GstOmapFbSink))

typedef struct GstXContext GstXContext;
typedef struct GstXWindow GstXWindow;
typedef struct GstOmapFbSink GstOmapFbSink;
typedef struct GstOmapFbSinkClass GstOmapFbSinkClass;

/**
 * GstXWindow:
 * @win: the Window ID of this X11 window
 * @width: the width in pixels of Window @win
 * @height: the height in pixels of Window @win
 * @internal: used to remember if Window @win was created internally or passed
 * through the #GstXOverlay interface
 * @gc: the Graphical Context of Window @win
 *
 * Structure used to store informations about a Window.
 */
struct GstXWindow {
    Window win;
    gint width, height;
    gboolean internal;
    GC gc;

    gint wx, wy;
};

/**
 * GstXContext:
 * @disp: the X11 Display of this context
 * @screen: the default Screen of Display @disp
 * @screen_num: the Screen number of @screen
 * @visual: the default Visual of Screen @screen
 * @root: the root Window of Display @disp
 * @white: the value of a white pixel on Screen @screen
 * @black: the value of a black pixel on Screen @screen
 * @depth: the color depth of Display @disp
 * @bpp: the number of bits per pixel on Display @disp
 * @endianness: the endianness of image bytes on Display @disp
 * @width: the width in pixels of Display @disp
 * @height: the height in pixels of Display @disp
 *
 * Structure used to store various informations collected/calculated for a
 * Display.
 */
struct GstXContext {
    Display *disp;
    Screen *screen;
    gint screen_num;

    Visual *visual;
    Window root;

    gint depth;
    gint bpp;

    gint width, height;
};

struct GstOmapFbSink
{
    GstVideoSink videosink;

    struct fb_fix_screeninfo fixinfo;
    struct fb_var_screeninfo varinfo;
    struct fb_var_screeninfo overlay_info;
    struct omapfb_mem_info mem_info;
    struct omapfb_plane_info plane_info;

    int overlay_fd;
    unsigned char *framebuffer;
    gboolean enabled;

    GMutex *x_lock;
    GMutex *flow_lock;

    GstXContext *xcontext;
    GstXWindow *xwindow;

    gulong colorKey;
    char *display_name;
    GThread *event_thread;

    void *buffer;
    guint buffer_size;
    guint image_format;

    gboolean row_skip;
    gboolean handle_events;
    gboolean running;
};

struct GstOmapFbSinkClass
{
    GstVideoSinkClass parent_class;
};

GType gst_omapfbsink_get_type (void);

G_END_DECLS

#endif /* GST_OMAPFB_H */
